#include "cellulamainwindow.h"

CellulaMainWindow::CellulaMainWindow()
{
    createAction();
    createMenu();
    createToolbar();
    translateGUI();

    option_dialog = new Options(this);

    QGLFormat format(QGL::DoubleBuffer);
    format.setSamples(32);
    gl_widget = new GLwidget(format, this);
    setCentralWidget(gl_widget);
    setGeometry(0,0, 800, 600);
    render_timer = new QTimer(this);
    connect(render_timer, SIGNAL(timeout()), gl_widget, SLOT(updateGL()));
    render_timer->setInterval(1000);
    render_timer->start();
    connect(actionNew, SIGNAL(activated()), this, SLOT(showOptionDialog()));
    connect(actionRun, SIGNAL(activated()), this, SLOT(startSimulation()));

    machine = 0;
    createCLMachine();
}

CellulaMainWindow::~CellulaMainWindow()
{
    //dtor
}

void CellulaMainWindow::createMenu()
{
}

void CellulaMainWindow::createToolbar()
{
    main_toolbar = new QToolBar(trUtf8("Hlavný panel nástrojov"), this);
    main_toolbar->addAction(actionNew);
    main_toolbar->addAction(actionOpen);
    main_toolbar->addAction(actionRun);
    main_toolbar->addAction(actionPause);
    addToolBar(main_toolbar);
}

void CellulaMainWindow::translateGUI()
{
    setWindowTitle(trUtf8("Cellula"));
}

void CellulaMainWindow::createAction()
{
    //QCleanlooksStyle cur_style;
    //QIcon icon = cur_style.standardIcon(QStyle::SP_DirHomeIcon);
    actionNew = new QAction(trUtf8("Nový model"), this);
    actionOpen = new QAction(trUtf8("Otvor model"), this);
    actionRun = new QAction(trUtf8("Spusti simuláciu"), this);
    actionPause = new QAction(trUtf8("Pozastav simuláciu"), this);
}

void CellulaMainWindow::showOptionDialog()
{
    option_dialog->show();
}

void CellulaMainWindow::createCLMachine()
{
    delete machine;
    gl_widget->makeCurrent();
    machine = new Machine((cl_context_properties)glXGetCurrentContext(), (cl_context_properties)glXGetCurrentDisplay(), option_dialog->getPlatform(), option_dialog->getDevices());
    Model *model = new SynchronousModel(machine, "");
    QImage img(model->image(), 256, 256, QImage::Format_ARGB32);
    cout << img.save("img.png") << endl;

    space = new CellSpace(model, 256, 256);
}

void CellulaMainWindow::startSimulation()
{
    space->run(10);
    machine->finish();
}

void CellulaMainWindow::stopSimulation()
{
}
