#include "machine.h"
#include "cellspace.h"
#include <CL/cl_gl.h>

Machine::Machine(int platform_index, cl_device_v devices)
{
    cl_platform_v platform_ids = getPlatforms();

    cl_context_properties cl_properties[] = { CL_CONTEXT_PLATFORM, (cl_context_properties)platform_ids[platform_index], 0 };
    init(cl_properties, devices);
}

Machine::Machine(cl_context_properties gl_context, cl_context_properties display, int platform_index, cl_device_v devices)
{
    if(gl_context == 0 || display == 0)cerr << "Nie je inicializovane OpenGL" << endl;

    cl_platform_v platform_ids = getPlatforms();

    cl_context_properties cl_properties[] = { CL_CONTEXT_PLATFORM, (cl_context_properties)platform_ids[platform_index],
        CL_GL_CONTEXT_KHR, gl_context, CL_GLX_DISPLAY_KHR, display, 0 };

    init(cl_properties, devices);
}

void Machine::init(cl_context_properties *cl_properties, cl_device_v &devices)
{
    cl_int err_code;
    if(devices.empty())clCreateContextFromType(cl_properties, CL_DEVICE_TYPE_ALL, NULL, NULL, &err_code);
    else context = clCreateContext(cl_properties, devices.size(), &devices[0], NULL, NULL, &err_code);

    if(err_code){ cerr << "Nepodarilo sa inicializovat OpenCL "; clGetError(err_code); }
    else cout << "Inicializujem OpenCL" << endl;
    device = devices;
    for(cl_device_v::iterator i = device.begin(); i != device.end();i++)
    {
        queues.push_back(clCreateCommandQueue(context, *i, NULL, &err_code));
        if(err_code)
        {
            cerr << "Nepodarilo sa vytvorit OpenCL prikazove fronty "; clGetError(err_code);
            break;
        }
    }
    local_work_size = 64;
}

Machine::~Machine()
{
    for(vector<cl_command_queue>::iterator i = queues.begin(); i != queues.end(); i++)
        clReleaseCommandQueue(*i);
    for(vector<Model*>::iterator i =  models.begin(); i != models.end(); i++)
        delete *i;
    clReleaseContext(context);
}

cl_platform_v Machine::getPlatforms()
{
    cl_int err_code;
    cl_platform_v platform_ids;
    cl_uint num_entries;
    err_code = clGetPlatformIDs(0, NULL, &num_entries);
    if(err_code)return platform_ids;
    platform_ids.resize(num_entries);
    err_code = clGetPlatformIDs(num_entries, &platform_ids[0], NULL);
    return platform_ids;
}

cl_device_v Machine::getDevices(cl_platform_id platform_id, cl_device_type type)
{
    cl_device_v devices;
    cl_uint num_entries;
    cl_int err_code = clGetDeviceIDs(platform_id, type, 0, NULL, &num_entries);
    if(err_code)return devices;
    devices.resize(num_entries);
    clGetDeviceIDs(platform_id, type, num_entries, &devices[0], NULL);
    return devices;
}

void Machine::addChild(Model *child)
{
    models.push_back(child);
}

void Machine::removeChild(Model *child)
{
    for(vector<Model*>::iterator i = models.begin(); i != models.end();i++)
        if(*i == child)
        {
            models.erase(i);
            break;
        }
}

void Machine::run(cl_kernel kernel, size_t w, size_t h)
{
    if(queues.size()==0)return;
    cl_int err_code;
    size_t global_work[] = { w, h };
    size_t local_work[] = { 1, 1 };
    err_code = clEnqueueNDRangeKernel(queues[0], kernel, 2, NULL, global_work, local_work, 0, NULL, NULL);
    if(err_code)clGetError(err_code);
}

cl_context Machine::getContext()
{
    return context;
}

void Machine::finish()
{
    for(vector<cl_command_queue>::iterator i = queues.begin(); i != queues.end();i++)
        clFinish(*i);
}
