#include "model.h"
#include "machine.h"

Model::Model(Machine *parent)
{
    this->parent = parent;
    parent->addChild(this);
}

Model::~Model()
{
    parent->removeChild(this);
    clReleaseKernel(kernel);
    clReleaseProgram(program);
}

void Model::addChild(CellSpace *child)
{
    patterns.push_back(child);
}

void Model::removeChild(CellSpace *child)
{
    for(vector<CellSpace*>::iterator i = patterns.begin(); i != patterns.end();i++)
        if(*i == child)patterns.erase(i);
}

unsigned char* Model::image()
{
    cl_image_format format;
    format.image_channel_order = CL_ARGB;
    format.image_channel_data_type = CL_UNORM_INT8;
    cl_int err_code;
    unsigned char *data = new unsigned char[256*256*4];
    unsigned char *img = new unsigned char[256*256*4];
    for(int i=0;i<256;i++)
        for(int o=0;o<256;o++)
        {
            img[i*256*4+o*4] = (i%1)*255;
            img[i*256*4+o*4+1] = i;
            img[i*256*4+o*4+2] = o;
            img[i*256*4+o*4+3] = (o%1)*255;
        }

    cl_mem buf = clCreateBuffer(parent->getContext(), CL_MEM_READ_WRITE, 256*256*4, NULL, &err_code);
    cl_mem image = clCreateImage2D(parent->getContext(), CL_MEM_READ_WRITE | CL_MEM_COPY_HOST_PTR, &format, 256, 256, 0, img, &err_code);
    if(err_code)cerr << "Nepodarilo sa vytvorit obrazok" << endl;
    cl_kernel image_kernel = clCreateKernel(program, "image_write", &err_code);
    if(err_code)cerr << "Nepodarilo sa vytvorit image kernel" << endl;
    err_code = clSetKernelArg(image_kernel, 0, sizeof(cl_mem), &buf);
    err_code |= clSetKernelArg(image_kernel, 1, sizeof(cl_mem), &image);
    if(err_code)cerr << "Nepodarilo sa nasatvit argument" << endl;
    parent->run(image_kernel, 256, 256);
    parent->finish();
    size_t offset[] = {0, 0, 0};
    size_t region[] = {256, 256, 1};

    err_code = clEnqueueReadImage(parent->queues[0], image, CL_TRUE, offset, region, 0, 0, data, 0, NULL, NULL);
    //err_code = clEnqueueReadBuffer(parent->queues[0], buf, CL_TRUE, 0, 256*256*4, data, 0, NULL, NULL);
    if(err_code){cerr << "nepodarilo sa nacitat obrazok" << endl; clGetError(err_code); }
    parent->finish();
    return data;
}

/*void Model::run(CellSpace *pattern, int which)
{

}*/

cl_int Model::build(string source, string options)
{
    cl_int err_code;
    const char *s = source.c_str();
    size_t len[] = { source.length() };
    program = clCreateProgramWithSource(parent->getContext(), 1, &s, len, &err_code);
    if(err_code){ cerr << "Nepodarilo sa vytvorit program"; clGetError(err_code); return err_code; }
    err_code = clBuildProgram(program, 0, NULL, options.c_str(), NULL, NULL);
    if(err_code){ cerr << "Nepodarilo sa skompilovat program" << endl << getBuildLog(program) << endl; return err_code; }
    kernel = clCreateKernel(program, "takeStep", &err_code);
    if(err_code){ cerr << "Nepodarilo sa vytvorit kernel"; clGetError(err_code); return err_code; }
    return err_code;
}
